
/* 350 glyphs, generated on 2012-04-06 22:56:08; C=10; R=35; W=900; H=3150; PREFIX=glyph2x- */
.btn-large [class^="glyph2x-"] {
  margin-top: 1px;
}

.btn-small [class^="glyph2x-"] {
  margin-top: -1px;
}

.nav-list [class^="glyph2x-"] {
  margin-right: 2px;
}

[class^="glyph2x-"],
[class*=" glyph2x-"] {
  display: inline-block;
  width: 90px;
  height: 90px;
  line-height: 90px;
  vertical-align: bottom;
  background-image: url("glyph2x-dark.png");
  background-position: 90px 90px;
  background-repeat: no-repeat;
  *margin-right: .3em;
}
[class^="glyph2x-"]:last-child,
[class*=" glyph2x-"]:last-child {
  *margin-left: 0;
}

.glyph2x-white {
  background-image: url("glyph2x-white.png");
}
.glyph2x-glass {
	background-position: 0 0;
}
.glyph2x-leaf {
	background-position: -90px 0;
}
.glyph2x-dog {
	background-position: -180px 0;
}
.glyph2x-user {
	background-position: -270px 0;
}
.glyph2x-girl {
	background-position: -360px 0;
}
.glyph2x-car {
	background-position: -450px 0;
}
.glyph2x-user-add {
	background-position: -540px 0;
}
.glyph2x-user-remove {
	background-position: -630px 0;
}
.glyph2x-film {
	background-position: -720px 0;
}
.glyph2x-magic {
	background-position: -810px 0;
}
.glyph2x-envelope {
	background-position: 0 -90px;
}
.glyph2x-camera {
	background-position: -90px -90px;
}
.glyph2x-heart {
	background-position: -180px -90px;
}
.glyph2x-beach-umbrella {
	background-position: -270px -90px;
}
.glyph2x-train {
	background-position: -360px -90px;
}
.glyph2x-print {
	background-position: -450px -90px;
}
.glyph2x-bin {
	background-position: -540px -90px;
}
.glyph2x-music {
	background-position: -630px -90px;
}
.glyph2x-note {
	background-position: -720px -90px;
}
.glyph2x-cogwheel {
	background-position: -810px -90px;
}
.glyph2x-home {
	background-position: 0 -180px;
}
.glyph2x-snowflake {
	background-position: -90px -180px;
}
.glyph2x-fire {
	background-position: -180px -180px;
}
.glyph2x-cogwheels {
	background-position: -270px -180px;
}
.glyph2x-parents {
	background-position: -360px -180px;
}
.glyph2x-binoculars {
	background-position: -450px -180px;
}
.glyph2x-road {
	background-position: -540px -180px;
}
.glyph2x-search {
	background-position: -630px -180px;
}
.glyph2x-cars {
	background-position: -720px -180px;
}
.glyph2x-notes-2 {
	background-position: -810px -180px;
}
.glyph2x-pencil {
	background-position: 0 -270px;
}
.glyph2x-bus {
	background-position: -90px -270px;
}
.glyph2x-wifi-alt {
	background-position: -180px -270px;
}
.glyph2x-luggage {
	background-position: -270px -270px;
}
.glyph2x-old-man {
	background-position: -360px -270px;
}
.glyph2x-woman {
	background-position: -450px -270px;
}
.glyph2x-file {
	background-position: -540px -270px;
}
.glyph2x-credit {
	background-position: -630px -270px;
}
.glyph2x-airplane {
	background-position: -720px -270px;
}
.glyph2x-notes {
	background-position: -810px -270px;
}
.glyph2x-stats {
	background-position: 0 -360px;
}
.glyph2x-charts {
	background-position: -90px -360px;
}
.glyph2x-pie-chart {
	background-position: -180px -360px;
}
.glyph2x-group {
	background-position: -270px -360px;
}
.glyph2x-keys {
	background-position: -360px -360px;
}
.glyph2x-calendar {
	background-position: -450px -360px;
}
.glyph2x-router {
	background-position: -540px -360px;
}
.glyph2x-camera-small {
	background-position: -630px -360px;
}
.glyph2x-dislikes {
	background-position: -720px -360px;
}
.glyph2x-star {
	background-position: -810px -360px;
}
.glyph2x-link {
	background-position: 0 -450px;
}
.glyph2x-eye-open {
	background-position: -90px -450px;
}
.glyph2x-eye-close {
	background-position: -180px -450px;
}
.glyph2x-alarm {
	background-position: -270px -450px;
}
.glyph2x-clock {
	background-position: -360px -450px;
}
.glyph2x-stopwatch {
	background-position: -450px -450px;
}
.glyph2x-projector {
	background-position: -540px -450px;
}
.glyph2x-history {
	background-position: -630px -450px;
}
.glyph2x-truck {
	background-position: -720px -450px;
}
.glyph2x-cargo {
	background-position: -810px -450px;
}
.glyph2x-compass {
	background-position: 0 -540px;
}
.glyph2x-keynote {
	background-position: -90px -540px;
}
.glyph2x-attach {
	background-position: -180px -540px;
}
.glyph2x-power {
	background-position: -270px -540px;
}
.glyph2x-lightbulb {
	background-position: -360px -540px;
}
.glyph2x-tag {
	background-position: -450px -540px;
}
.glyph2x-tags {
	background-position: -540px -540px;
}
.glyph2x-cleaning {
	background-position: -630px -540px;
}
.glyph2x-ruller {
	background-position: -720px -540px;
}
.glyph2x-gift {
	background-position: -810px -540px;
}
.glyph2x-umbrella {
	background-position: 0 -630px;
}
.glyph2x-book {
	background-position: -90px -630px;
}
.glyph2x-bookmark {
	background-position: -180px -630px;
}
.glyph2x-signal {
	background-position: -270px -630px;
}
.glyph2x-cup {
	background-position: -360px -630px;
}
.glyph2x-stroller {
	background-position: -450px -630px;
}
.glyph2x-headphones {
	background-position: -540px -630px;
}
.glyph2x-headset {
	background-position: -630px -630px;
}
.glyph2x-warning-sign {
	background-position: -720px -630px;
}
.glyph2x-signal {
	background-position: -810px -630px;
}
.glyph2x-retweet {
	background-position: 0 -720px;
}
.glyph2x-refresh {
	background-position: -90px -720px;
}
.glyph2x-roundabout {
	background-position: -180px -720px;
}
.glyph2x-random {
	background-position: -270px -720px;
}
.glyph2x-heat {
	background-position: -360px -720px;
}
.glyph2x-repeat {
	background-position: -450px -720px;
}
.glyph2x-display {
	background-position: -540px -720px;
}
.glyph2x-log-book {
	background-position: -630px -720px;
}
.glyph2x-adress-book {
	background-position: -720px -720px;
}
.glyph2x-magnet {
	background-position: -810px -720px;
}
.glyph2x-table {
	background-position: 0 -810px;
}
.glyph2x-adjust {
	background-position: -90px -810px;
}
.glyph2x-tint {
	background-position: -180px -810px;
}
.glyph2x-crop {
	background-position: -270px -810px;
}
.glyph2x-vector-path-square {
	background-position: -360px -810px;
}
.glyph2x-vector-path-circle {
	background-position: -450px -810px;
}
.glyph2x-vector-path-polygon {
	background-position: -540px -810px;
}
.glyph2x-vector-path-line {
	background-position: -630px -810px;
}
.glyph2x-vector-path-curve {
	background-position: -720px -810px;
}
.glyph2x-vector-path-all {
	background-position: -810px -810px;
}
.glyph2x-font {
	background-position: 0 -900px;
}
.glyph2x-italic {
	background-position: -90px -900px;
}
.glyph2x-bold {
	background-position: -180px -900px;
}
.glyph2x-text-underline {
	background-position: -270px -900px;
}
.glyph2x-text-strike {
	background-position: -360px -900px;
}
.glyph2x-text-height {
	background-position: -450px -900px;
}
.glyph2x-text-width {
	background-position: -540px -900px;
}
.glyph2x-text-resize {
	background-position: -630px -900px;
}
.glyph2x-left-indent {
	background-position: -720px -900px;
}
.glyph2x-right-indent {
	background-position: -810px -900px;
}
.glyph2x-align-left {
	background-position: 0 -990px;
}
.glyph2x-align-center {
	background-position: -90px -990px;
}
.glyph2x-align-right {
	background-position: -180px -990px;
}
.glyph2x-justify {
	background-position: -270px -990px;
}
.glyph2x-list {
	background-position: -360px -990px;
}
.glyph2x-text-smaller {
	background-position: -450px -990px;
}
.glyph2x-text-bigger {
	background-position: -540px -990px;
}
.glyph2x-embed {
	background-position: -630px -990px;
}
.glyph2x-embed-close {
	background-position: -720px -990px;
}
.glyph2x-adjust {
	background-position: -810px -990px;
}
.glyph2x-message-full {
	background-position: 0 -1080px;
}
.glyph2x-message-empty {
	background-position: -90px -1080px;
}
.glyph2x-message-in {
	background-position: -180px -1080px;
}
.glyph2x-message-out {
	background-position: -270px -1080px;
}
.glyph2x-message-plus {
	background-position: -360px -1080px;
}
.glyph2x-message-minus {
	background-position: -450px -1080px;
}
.glyph2x-message-ban {
	background-position: -540px -1080px;
}
.glyph2x-message-flag {
	background-position: -630px -1080px;
}
.glyph2x-message-lock {
	background-position: -720px -1080px;
}
.glyph2x-message-new {
	background-position: -810px -1080px;
}
.glyph2x-inbox {
	background-position: 0 -1170px;
}
.glyph2x-inbox-plus {
	background-position: -90px -1170px;
}
.glyph2x-inbox-minus {
	background-position: -180px -1170px;
}
.glyph2x-inbox-lock {
	background-position: -270px -1170px;
}
.glyph2x-inbox-in {
	background-position: -360px -1170px;
}
.glyph2x-inbox-out {
	background-position: -450px -1170px;
}
.glyph2x-computer-locked {
	background-position: -540px -1170px;
}
.glyph2x-computer-service {
	background-position: -630px -1170px;
}
.glyph2x-computer-proces {
	background-position: -720px -1170px;
}
.glyph2x-phone {
	background-position: -810px -1170px;
}
.glyph2x-database-lock {
	background-position: 0 -1260px;
}
.glyph2x-database-plus {
	background-position: -90px -1260px;
}
.glyph2x-database-minus {
	background-position: -180px -1260px;
}
.glyph2x-database-ban {
	background-position: -270px -1260px;
}
.glyph2x-folder-open {
	background-position: -360px -1260px;
}
.glyph2x-folder-plus {
	background-position: -450px -1260px;
}
.glyph2x-folder-minus {
	background-position: -540px -1260px;
}
.glyph2x-folder-lock {
	background-position: -630px -1260px;
}
.glyph2x-folder-flag {
	background-position: -720px -1260px;
}
.glyph2x-folder-new {
	background-position: -810px -1260px;
}
.glyph2x-check {
	background-position: 0 -1350px;
}
.glyph2x-edit {
	background-position: -90px -1350px;
}
.glyph2x-new-window {
	background-position: -180px -1350px;
}
.glyph2x-more-windows {
	background-position: -270px -1350px;
}
.glyph2x-show-big-thumbnails {
	background-position: -360px -1350px;
}
.glyph2x-show-thumbnails {
	background-position: -450px -1350px;
}
.glyph2x-show-thumbnails-with-lines {
	background-position: -540px -1350px;
}
.glyph2x-show-lines {
	background-position: -630px -1350px;
}
.glyph2x-playlist {
	background-position: -720px -1350px;
}
.glyph2x-picture {
	background-position: -810px -1350px;
}
.glyph2x-imac {
	background-position: 0 -1440px;
}
.glyph2x-macbook {
	background-position: -90px -1440px;
}
.glyph2x-ipad {
	background-position: -180px -1440px;
}
.glyph2x-iphone {
	background-position: -270px -1440px;
}
.glyph2x-iphone-transfer {
	background-position: -360px -1440px;
}
.glyph2x-iphone-exchange {
	background-position: -450px -1440px;
}
.glyph2x-ipod {
	background-position: -540px -1440px;
}
.glyph2x-ipod-shuffle {
	background-position: -630px -1440px;
}
.glyph2x-ear-plugs {
	background-position: -720px -1440px;
}
.glyph2x-albums {
	background-position: -810px -1440px;
}
.glyph2x-step-backward {
	background-position: 0 -1530px;
}
.glyph2x-fast-backward {
	background-position: -90px -1530px;
}
.glyph2x-rewind {
	background-position: -180px -1530px;
}
.glyph2x-play {
	background-position: -270px -1530px;
}
.glyph2x-pause {
	background-position: -360px -1530px;
}
.glyph2x-stop {
	background-position: -450px -1530px;
}
.glyph2x-forward {
	background-position: -540px -1530px;
}
.glyph2x-fast-forward {
	background-position: -630px -1530px;
}
.glyph2x-step-forward {
	background-position: -720px -1530px;
}
.glyph2x-eject {
	background-position: -810px -1530px;
}
.glyph2x-facetime-video {
	background-position: 0 -1620px;
}
.glyph2x-download-alt {
	background-position: -90px -1620px;
}
.glyph2x-mute {
	background-position: -180px -1620px;
}
.glyph2x-volume-down {
	background-position: -270px -1620px;
}
.glyph2x-volume-up {
	background-position: -360px -1620px;
}
.glyph2x-screenshot {
	background-position: -450px -1620px;
}
.glyph2x-move {
	background-position: -540px -1620px;
}
.glyph2x-more {
	background-position: -630px -1620px;
}
.glyph2x-brightness-reduce {
	background-position: -720px -1620px;
}
.glyph2x-brightness-increase {
	background-position: -810px -1620px;
}
.glyph2x-circle-plus {
	background-position: 0 -1710px;
}
.glyph2x-circle-minus {
	background-position: -90px -1710px;
}
.glyph2x-circle-remove {
	background-position: -180px -1710px;
}
.glyph2x-circle-ok {
	background-position: -270px -1710px;
}
.glyph2x-circle-question-mark {
	background-position: -360px -1710px;
}
.glyph2x-circle-info {
	background-position: -450px -1710px;
}
.glyph2x-circle-exclamation-mark {
	background-position: -540px -1710px;
}
.glyph2x-remove {
	background-position: -630px -1710px;
}
.glyph2x-ok {
	background-position: -720px -1710px;
}
.glyph2x-ban {
	background-position: -810px -1710px;
}
.glyph2x-download {
	background-position: 0 -1800px;
}
.glyph2x-upload {
	background-position: -90px -1800px;
}
.glyph2x-shopping-cart {
	background-position: -180px -1800px;
}
.glyph2x-lock {
	background-position: -270px -1800px;
}
.glyph2x-unlock {
	background-position: -360px -1800px;
}
.glyph2x-electricity {
	background-position: -450px -1800px;
}
.glyph2x-ok-2 {
	background-position: -540px -1800px;
}
.glyph2x-remove-2 {
	background-position: -630px -1800px;
}
.glyph2x-cart-out {
	background-position: -720px -1800px;
}
.glyph2x-cart-in {
	background-position: -810px -1800px;
}
.glyph2x-left-arrow {
	background-position: 0 -1890px;
}
.glyph2x-right-arrow {
	background-position: -90px -1890px;
}
.glyph2x-down-arrow {
	background-position: -180px -1890px;
}
.glyph2x-up-arrow {
	background-position: -270px -1890px;
}
.glyph2x-resize-small {
	background-position: -360px -1890px;
}
.glyph2x-resize-full {
	background-position: -450px -1890px;
}
.glyph2x-circle-arrow-left {
	background-position: -540px -1890px;
}
.glyph2x-circle-arrow-right {
	background-position: -630px -1890px;
}
.glyph2x-circle-arrow-right {
	background-position: -720px -1890px;
}
.glyph2x-circle-arrow-right {
	background-position: -810px -1890px;
}
.glyph2x-play-button {
	background-position: 0 -1980px;
}
.glyph2x-unshare {
	background-position: -90px -1980px;
}
.glyph2x-share {
	background-position: -180px -1980px;
}
.glyph2x-thin-right-arrow {
	background-position: -270px -1980px;
}
.glyph2x-thin-arrow-left {
	background-position: -360px -1980px;
}
.glyph2x-bluetooth {
	background-position: -450px -1980px;
}
.glyph2x-euro {
	background-position: -540px -1980px;
}
.glyph2x-usd {
	background-position: -630px -1980px;
}
.glyph2x-bp {
	background-position: -720px -1980px;
}
.glyph2x-retweet-2 {
	background-position: -810px -1980px;
}
.glyph2x-moon {
	background-position: 0 -2070px;
}
.glyph2x-sun {
	background-position: -90px -2070px;
}
.glyph2x-cloud {
	background-position: -180px -2070px;
}
.glyph2x-direction {
	background-position: -270px -2070px;
}
.glyph2x-brush {
	background-position: -360px -2070px;
}
.glyph2x-pen {
	background-position: -450px -2070px;
}
.glyph2x-zoom-in {
	background-position: -540px -2070px;
}
.glyph2x-zoom-out {
	background-position: -630px -2070px;
}
.glyph2x-pin {
	background-position: -720px -2070px;
}
.glyph2x-riflescope {
	background-position: -810px -2070px;
}
.glyph2x-rotation-lock {
	background-position: 0 -2160px;
}
.glyph2x-flash {
	background-position: -90px -2160px;
}
.glyph2x-google-maps {
	background-position: -180px -2160px;
}
.glyph2x-anchor {
	background-position: -270px -2160px;
}
.glyph2x-conversation {
	background-position: -360px -2160px;
}
.glyph2x-chat {
	background-position: -450px -2160px;
}
.glyph2x-male {
	background-position: -540px -2160px;
}
.glyph2x-female {
	background-position: -630px -2160px;
}
.glyph2x-asterisk {
	background-position: -720px -2160px;
}
.glyph2x-divide {
	background-position: -810px -2160px;
}
.glyph2x-snorkel-diving {
	background-position: 0 -2250px;
}
.glyph2x-scuba-diving {
	background-position: -90px -2250px;
}
.glyph2x-oxygen-bottle {
	background-position: -180px -2250px;
}
.glyph2x-fins {
	background-position: -270px -2250px;
}
.glyph2x-fishes {
	background-position: -360px -2250px;
}
.glyph2x-boat {
	background-position: -450px -2250px;
}
.glyph2x-delete-point {
	background-position: -540px -2250px;
}
.glyph2x-sheriffs--star {
	background-position: -630px -2250px;
}
.glyph2x-qrcode {
	background-position: -720px -2250px;
}
.glyph2x-barcode {
	background-position: -810px -2250px;
}
.glyph2x-pool {
	background-position: 0 -2340px;
}
.glyph2x-buoy {
	background-position: -90px -2340px;
}
.glyph2x-spade {
	background-position: -180px -2340px;
}
.glyph2x-bank {
	background-position: -270px -2340px;
}
.glyph2x-vcard {
	background-position: -360px -2340px;
}
.glyph2x-electrical-plug {
	background-position: -450px -2340px;
}
.glyph2x-flag {
	background-position: -540px -2340px;
}
.glyph2x-credit-card {
	background-position: -630px -2340px;
}
.glyph2x-keyboard-wireless {
	background-position: -720px -2340px;
}
.glyph2x-keyboard-wired {
	background-position: -810px -2340px;
}
.glyph2x-shield {
	background-position: 0 -2430px;
}
.glyph2x-ring {
	background-position: -90px -2430px;
}
.glyph2x-cake {
	background-position: -180px -2430px;
}
.glyph2x-drink {
	background-position: -270px -2430px;
}
.glyph2x-beer {
	background-position: -360px -2430px;
}
.glyph2x-fast-food {
	background-position: -450px -2430px;
}
.glyph2x-cutlery {
	background-position: -540px -2430px;
}
.glyph2x-pizza {
	background-position: -630px -2430px;
}
.glyph2x-birthday-cake {
	background-position: -720px -2430px;
}
.glyph2x-tablet {
	background-position: -810px -2430px;
}
.glyph2x-settings {
	background-position: 0 -2520px;
}
.glyph2x-bullets {
	background-position: -90px -2520px;
}
.glyph2x-cardio {
	background-position: -180px -2520px;
}
.glyph2x-t-shirt {
	background-position: -270px -2520px;
}
.glyph2x-pants {
	background-position: -360px -2520px;
}
.glyph2x-sweater {
	background-position: -450px -2520px;
}
.glyph2x-fabric {
	background-position: -540px -2520px;
}
.glyph2x-leather {
	background-position: -630px -2520px;
}
.glyph2x-scissors {
	background-position: -720px -2520px;
}
.glyph2x-podium {
	background-position: -810px -2520px;
}
.glyph2x-skull {
	background-position: 0 -2610px;
}
.glyph2x-celebration {
	background-position: -90px -2610px;
}
.glyph2x-tea-kettle {
	background-position: -180px -2610px;
}
.glyph2x-french-press {
	background-position: -270px -2610px;
}
.glyph2x-coffe-cup {
	background-position: -360px -2610px;
}
.glyph2x-pot {
	background-position: -450px -2610px;
}
.glyph2x-grater {
	background-position: -540px -2610px;
}
.glyph2x-kettle {
	background-position: -630px -2610px;
}
.glyph2x-hospital {
	background-position: -720px -2610px;
}
.glyph2x-hospital-h {
	background-position: -810px -2610px;
}
.glyph2x-microphone {
	background-position: 0 -2700px;
}
.glyph2x-webcam {
	background-position: -90px -2700px;
}
.glyph2x-temple-christianity-church {
	background-position: -180px -2700px;
}
.glyph2x-temple-islam {
	background-position: -270px -2700px;
}
.glyph2x-temple-hindu {
	background-position: -360px -2700px;
}
.glyph2x-temple-buddhist {
	background-position: -450px -2700px;
}
.glyph2x-electrical-socket-eu {
	background-position: -540px -2700px;
}
.glyph2x-electrical-socket-us {
	background-position: -630px -2700px;
}
.glyph2x-bomb {
	background-position: -720px -2700px;
}
.glyph2x-comments {
	background-position: -810px -2700px;
}
.glyph2x-flower {
	background-position: 0 -2790px;
}
.glyph2x-baseball {
	background-position: -90px -2790px;
}
.glyph2x-rugby {
	background-position: -180px -2790px;
}
.glyph2x-ax {
	background-position: -270px -2790px;
}
.glyph2x-table-tennis {
	background-position: -360px -2790px;
}
.glyph2x-bowling {
	background-position: -450px -2790px;
}
.glyph2x-tree-conifer {
	background-position: -540px -2790px;
}
.glyph2x-tree-deciduous {
	background-position: -630px -2790px;
}
.glyph2x-more-items {
	background-position: -720px -2790px;
}
.glyph2x-sort {
	background-position: -810px -2790px;
}
.glyph2x-facebook {
	background-position: 0 -2880px;
}
.glyph2x-twitter-t {
	background-position: -90px -2880px;
}
.glyph2x-twitter {
	background-position: -180px -2880px;
}
.glyph2x-buzz {
	background-position: -270px -2880px;
}
.glyph2x-vimeo {
	background-position: -360px -2880px;
}
.glyph2x-flickr {
	background-position: -450px -2880px;
}
.glyph2x-last-fm {
	background-position: -540px -2880px;
}
.glyph2x-rss {
	background-position: -630px -2880px;
}
.glyph2x-skype {
	background-position: -720px -2880px;
}
.glyph2x-e-mail {
	background-position: -810px -2880px;
}
.glyph2x-instapaper {
	background-position: 0 -2970px;
}
.glyph2x-evernote {
	background-position: -90px -2970px;
}
.glyph2x-xing {
	background-position: -180px -2970px;
}
.glyph2x-zootool {
	background-position: -270px -2970px;
}
.glyph2x-dribbble {
	background-position: -360px -2970px;
}
.glyph2x-deviantart {
	background-position: -450px -2970px;
}
.glyph2x-read-it-later {
	background-position: -540px -2970px;
}
.glyph2x-linked-in {
	background-position: -630px -2970px;
}
.glyph2x-forrst {
	background-position: -720px -2970px;
}
.glyph2x-pinboard {
	background-position: -810px -2970px;
}
.glyph2x-behance {
	background-position: 0 -3060px;
}
.glyph2x-github {
	background-position: -90px -3060px;
}
.glyph2x-youtube {
	background-position: -180px -3060px;
}
.glyph2x-skitch {
	background-position: -270px -3060px;
}
.glyph2x-4square {
	background-position: -360px -3060px;
}
.glyph2x-quora {
	background-position: -450px -3060px;
}
.glyph2x-google-plus {
	background-position: -540px -3060px;
}
.glyph2x-spootify {
	background-position: -630px -3060px;
}
.glyph2x-stumbleupon {
	background-position: -720px -3060px;
}
.glyph2x-readability {
	background-position: -810px -3060px;
}
