<?php

namespace Sabre\VObject\Property\ICalendar;

/**
 * DateTime property.
 *
 * This object represents DATE values, as defined here:
 *
 * http://tools.ietf.org/html/rfc5545#section-3.3.5
 *
 * @copyright Copyright (C) fruux GmbH (https://fruux.com/)
 * @author Evert Pot (http://evertpot.com/)
 * @license http://sabre.io/license/ Modified BSD License
 */
class Date extends DateTime
{
}
