{% autoescape false %}
<!DOCTYPE html>
<html lang="en">
	<head>
		<meta charset="utf-8" />
		<title>{{ pagetitle }}</title>
		<base href="{{ baseurl }}" />
		<meta name="viewport" content="width=device-width, initial-scale=1.0" />

		<!-- Le styles -->
		<link href="res/core/TwitterBootstrap/css/bootstrap.css" rel="stylesheet" />
		<link href="res/core/BaikalAdmin/GlyphiconsPro/glyphpro.css" rel="stylesheet" />
		<link href="res/core/BaikalAdmin/GlyphiconsPro/glyphpro-2x.css" rel="stylesheet" />
		<link href="res/core/BaikalAdmin/Templates/Page/style.css" rel="stylesheet" />
		<link href="res/core/TwitterBootstrap/css/bootstrap-responsive.css" rel="stylesheet" />

		{{ head }}
	</head>
	<body>
		{{ navbar }}

		<div class="container">
			{{ Payload }}
		</div> <!-- /container -->

		<!-- Le javascript
		================================================== -->
		<!-- Placed at the end of the document so the pages load faster -->
		<script src="res/core/TwitterBootstrap/js/jquery-3.1.0.min.js"></script>
		<script src="res/core/TwitterBootstrap/js/jquery.color-2.2.0.min.js"></script>
		<script src="res/core/TwitterBootstrap/js/bootstrap.min.js"></script>
		<script src="res/core/BaikalAdmin/main.js"></script>
		{{ javascript }}
	</body>
</html>
{% endautoescape %}
