<?php

declare(strict_types=1);

namespace Sabre\Xml;

use Exception;

/**
 * This is a base exception for any exception related to parsing xml files.
 *
 * @copyright Copyright (C) 2009-2015 fruux GmbH (https://fruux.com/).
 * @author Evert Pot (http://evertpot.com/)
 * @license http://sabre.io/license/ Modified BSD License
 */
class ParseException extends Exception
{
}
