{% autoescape false %}
<div class="navbar navbar-inverse navbar-fixed-top">
	<div class="navbar-inner">
		<div class="container">
			<a class="btn btn-navbar" data-toggle="collapse" data-target=".nav-collapse">
				<span class="icon-bar"></span>
				<span class="icon-bar"></span>
				<span class="icon-bar"></span>
			</a>
			<a class="brand" href="{{ homelink }}"><img style="vertical-align: text-top; line-height: 20px;" src="res/core/BaikalAdmin/Templates/Page/baikal-text-20.png" /> Web Admin</a>
			<div class="nav-collapse">
				<ul class="nav">
					<li class="{{ activehome }}"> <a href="{{ homelink }}">Dashboard</a></li>
					<li class="{{ activeusers }}"> <a href="{{ userslink }}">Users and resources</a></li>
					<li class="{{ activesettingsstandard }}"> <a href="{{ settingsstandardlink }}">System Settings</a></li>
					<li class="{{ activesettingsdatabase }}"> <a href="{{ settingsdatabaselink }}">Database settings</a></li>
					<li> <a href="{{ logoutlink }}"><i class="icon-eject icon-white"></i> Logout</a></li>
				</ul>
			</div>
		</div>
	</div>
</div>
{% endautoescape %}
