/* generics */

body {
	padding-top: 60px; /* 60px to make the container go all the way to the bottom of the topbar */
}

.table thead th {
	background-color: #777;
	color: white;
}

.table-striped tbody tr:nth-child(even) td, .table-striped tbody tr:nth-child(even) th {
	background-color: rgb(240, 240, 240);
}

table .no-border-left { border-left: none !important;}
table p {
	margin-bottom: 0;
}

p.lead { line-height: 40px;}


/* Jumbotrons
-------------------------------------------------- */
.jumbotron {
  position: relative;
}
.jumbotron h1 {
  font-size: 40px;
  font-weight: bold;
  letter-spacing: -1px;
  line-height: 90px;
}
.jumbotron p {
  margin-bottom: 18px;
  font-weight: 300;
}
.jumbotron .btn-large {
  font-size: 20px;
  font-weight: normal;
  padding: 14px 24px;
  margin-right: 10px;
  -webkit-border-radius: 6px;
     -moz-border-radius: 6px;
          border-radius: 6px;
}
.jumbotron .btn-large small {
  font-size: 14px;
}

@media (max-width: 550px) {
	.jumbotron h1 {
		font-size: 20px;
		font-weight: bold;
		letter-spacing: -1px;
		line-height: 20px;
	}
	
	p.lead {
		font-size: 14px;
		line-height: 14px;
	}
	
	[class^="glyph2x-"],
	[class*=" glyph2x-"] {
		display: none;
	}
}

/* Address books */
table.addressbooks .col-displayname { width: 20%;}
table.addressbooks .col-description { width: 40%;}
table.addressbooks .col-actions { width: 40%;}

/* Calendars */
table.calendars .col-displayname { width: 20%;}
table.calendars .col-description { width: 40%;}
table.calendars .col-actions { width: 40%;}

/* Users */
table.users .col-id { width: 2%;}
table.users .col-username { width: 45%;}

/* Loader */
.loader {
  display: inline-block;
  width: 64px;
  height: 64px;
}
.loader:after {
  content: " ";
  display: block;
  width: 46px;
  height: 46px;
  margin: 1px;
  border-radius: 50%;
  border: 5px solid #000;
  border-color: #000 transparent #000 transparent;
  animation: lds-dual-ring 1.2s linear infinite;
}
@keyframes lds-dual-ring {
  0% {
    transform: rotate(0deg);
  }
  100% {
    transform: rotate(360deg);
  }
}

.popover {
  max-width: 100%;
}

.copy-to-clipboard {
  cursor: pointer;
}

.copy-to-clipboard i {
  margin-left: 5px;
  opacity: 1;
}

.copy-to-clipboard:hover i {
  opacity: 0.5;
}
