{% autoescape false %}
<div class="navbar navbar-inverse navbar-fixed-top">
	<div class="navbar-inner">
		<div class="container">
			<a class="btn btn-navbar" data-toggle="collapse" data-target=".nav-collapse">
				<span class="icon-bar"></span>
				<span class="icon-bar"></span>
				<span class="icon-bar"></span>
			</a>
			<a class="brand" href="{{ homelink }}"><img style="vertical-align: text-top; line-height: 20px;" src="res/core/BaikalAdmin/Templates/Page/baikal-text-20.png" alt="baikal"> Server</a>
			<div class="nav-collapse">
				<ul class="nav">
					<li> <a href="admin/"><i class="icon-eject icon-white" style="transform: rotate(90deg)"></i> Login</a></li>
					<li> <a target="_blank" rel="noopener noreferrer" href="https://sabre.io/baikal/"><i class="icon-heart icon-white"></i> Get your own</a></li>
				</ul>
			</div>
		</div>
	</div>
</div>
{% endautoescape %}
