<?php

// autoload_namespaces.php @generated by Composer

$vendorDir = dirname(__DIR__);
$baseDir = dirname($vendorDir);

return array(
    'Twig_' => array($vendorDir . '/twig/twig/lib'),
    'Formal' => array($baseDir . '/Core/Frameworks'),
    'Flake' => array($baseDir . '/Core/Frameworks'),
    'Baikal' => array($baseDir . '/Core/Frameworks'),
);
