{% autoescape false %}
<header class="jumbotron subhead" id="overview">
	<h1><i class="glyph2x-magic"></i>Baïkal Database setup</h1>
	<p class="lead">Configure Baïkal Database.</p>
</header>

<a id="formid"></a>
{{ message }}
{{ form }}

{% endautoescape %}