{% autoescape false %}
<header class="jumbotron subhead" id="overview">
	<h1><i class="glyph2x-lock"></i>Authentication</h1>
	<p class="lead">Please authenticate to access Baïkal Web Admin.</p>
</header>

{{ message }}

<form class="form-horizontal" action="{{ actionurl }}" method="post" enctype="multipart/form-data">
	<input type="hidden" name="{{ submittedflagname }}" value="1" />
	<fieldset>
		<p>
			<label for="login">Login</label>
			<input type="text" id="login" name="login" value="{{ login }}">
		</p>

		<p>
			<label for="password">Password</label>
			<input type="password" id="password" name="password" value="{{ password }}">
		</p>
		
		<div class="form-actions">
			<button type="submit" class="btn btn-primary">Authenticate</button>
		</div>
	</fieldset>
</form>
{% endautoescape %}