{% autoescape false %}
<header class="jumbotron subhead" id="overview" style="padding-bottom:7em;">
	<h1><img style="height:150px; float:left;" src="res/core/Baikal/Images/logo-baikal.png" alt="baikal logo"> Baïkal
	<p class="lead">Lightweight CalDAV+CardDAV server.</p></h1>
</header>

{{ message }}

<section id="about">
	<div class="page-header">
		<h1>Baïkal is running alright.</h1>
	</div>
</section>
{% endautoescape %}
